//Macro developed by Cameron J. Nowell of Monash University (cameron.nowell@monash.edu) based orginal algorithm
//developed to measure worm and fly larvae movement for drug screening. 
//Code will take ND2 files captured on a Nikon microscope and measure the degree of movement
//
//Requirements - Standard install of Fiji and BAR plugin
//Assumptions
// - Data is captured using a monochrome camera on a Nikon microscope
// - All time segments are the same length
// - There is a baseline capture that two other captures are compared back to


fs=File.separator;
roiManager("reset");
run("Clear Results");

//build manual arrays for dialog pull down menus for scale and sampling window sizes
scaleItems=newArray(32,4,8,16,64,128);
windowItems=newArray(25,10,50,75,100);

//create dialog box to collect configuration requiremnts for the analysis
Dialog.create("Analysis Settings");
Dialog.addMessage("\n\nExperiment Details", 16);
Dialog.addString("Experiment Name", "Test");
Dialog.addString("Segment One Name", "One");
Dialog.addString("Segment Two Name", "Two");
Dialog.addString("Segment Three Name", "Three");
Dialog.addMessage("\n\nAnalysis Details", 16);
Dialog.addChoice("Scaling Level for Final Image", scaleItems);
Dialog.addChoice("Window Sampling Size for Heat Map Movie", windowItems);
Dialog.addNumber("Frames to Skip at Segment Start", 30);
Dialog.addNumber("Frames to Extract Per Segment", 150);
Dialog.show();

//store values from dialog box in custom varialbles
expName=Dialog.getString();
stimOne=Dialog.getString();
stimTwo=Dialog.getString();
stimThree=Dialog.getString();
scaleNumber=Dialog.getChoice();
windowSize=Dialog.getChoice();
segSkip=Dialog.getNumber();
segWidth=Dialog.getNumber();

//create array list to store image names in
stimList=newArray(stimOne,stimTwo,stimThree);


//prompt user to select first file in the in the time series
firstFile=File.openDialog("Select First File In Image Sequence");

run("Bio-Formats Macro Extensions");

run("Bio-Formats Importer", "open=["+firstFile+"] autoscale color_mode=Default rois_import=[ROI manager] specify_range view=Hyperstack stack_order=XYCZT t_begin=1 t_end=1 t_step=1");


//open first file, store dimensions and get timng data from meta data.
//open(firstFile);
nameStore=getTitle;
getDimensions(width, height, channels, slices, frames);
time=1;

//create output directories for path of opened file for ROIs, Heat Maps, Data Sheets and Movies
dir=File.getParent(firstFile)+fs;	
roiDir=dir+"ROIs"+fs;
File.makeDirectory(roiDir);
heatMapDir=dir+"Heat Maps"+fs;
File.makeDirectory(heatMapDir);
dataDir=dir+"Data"+fs;
File.makeDirectory(dataDir);
movieDir=dir+"Movies"+fs;
File.makeDirectory(movieDir);
roiPath=roiDir+"Tissue Outline.roi";

//scale the first frame of the time series based on the selected scale number
run("Scale...", "x=- y=- z=1.0 width="+(width/scaleNumber)+" height="+(height/scaleNumber)+" depth="+slices+" interpolation=None process create title=Scaled");
selectWindow("Scaled");
run("Set... ", "zoom=1600 x="+((width/scaleNumber)/2)+" y="+((height/scaleNumber)/2));

//set wand tool and ask user to click on the tissue to use it for a mask. 
setTool("wand");
run("Wand Tool...", "tolerance=4500 mode=Legacy");
waitForUser("Click Tissue");
roiManager("Add");
roiManager("Save", roiDir+expName+" - Tissue Outline.roi");

roiManager("reset");
selectWindow("Scaled");
run("Close");
run("Close All");

//generate data tables for recording data
tableTitle="[Raw Standard Deviation Values]";
if (isOpen("Raw Standard Deviation Values")){
	print("Table already open");}
	
	else{
		
		run("Table...", "name="+tableTitle+" width=1200 height=250");
		print(tableTitle, "\\Headings:Experiment Name\tStimulation Type\tScale Level\tPixel ID\tStandard Deviation");
		}

tableTitle2="[Standard Deviation Scoring]";
if (isOpen("Standard Deviation Scoring")){
	print("Table already open");}
	
	else{
		
		run("Table...", "name="+tableTitle2+" width=1200 height=250");
		print(tableTitle2, "\\Headings:Experiment Name\tStimulation Type\tScale Level\tTotal Pixels Measured\tCount >2 Std Deviations\tCount Between 1 and 2 Std Deviations\tCount Between 1 and 0 Std Deviations\tPercent >2 Std Deviations\tPercent Between 1 and 2 Std Deviations\tPercent Between 1 and 0 Std Deviations");
		}

tableTitle3="[Deviation Relative Change]";
if (isOpen("Deviation Relative Change")){
	print("Table already open");}
	
	else{
		
		run("Table...", "name="+tableTitle3+" width=1200 height=250");
		print(tableTitle3, "\\Headings:Experiment Name\tScale Level\tPixel ID\t"+stimList[1]+" Fold Change to "+stimList[0]+"\t"+stimList[2]+" Fold Change to "+stimList[0]+"\t"+stimList[2]+" Fold Change to "+stimList[1]+"\tFold Change Direction Relative to "+stimList[0]+"\tFold Change Direction Relative to Previous Stimulation");
		}

tableTitle4="[Deviation Relative Change - Rolling Average]";
if (isOpen("Deviation Relative Change - Rolling Average")){
	print("Table already open");}
	
	else{
		
		run("Table...", "name="+tableTitle4+" width=1200 height=250");
		print(tableTitle4, "\\Headings:Experiment Name\tScale Level\tPixel ID\t"+stimList[1]+" Fold Change to "+stimList[0]+"\t"+stimList[2]+" Fold Change to "+stimList[0]+"\t"+stimList[2]+" Fold Change to "+stimList[1]+"\tFold Change Direction Relative to "+stimList[0]+"\tFold Change Direction Relative to Previous Stimulation");
		}

run("Set Measurements...", "mean standard min limit redirect=None decimal=3");

//load image sequence and change image name to the user defined experiment name
print("Loading Image Sequence");
//open("C:/Users/camer/Desktop/Film2.tif");
run("Bio-Formats Importer", "open=["+firstFile+"] autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
rename(expName);
getDimensions(width, height, channels, slices, frames);
run("Properties...", "frame=1");

//reset ROI manager and results
roiManager("reset");
roiManager("Show All");
run("Clear Results");

//Use BAR plugin to find dark frames representing additions.
print("Finding Addition Points");

title="Plot Values";

run("Plot Z-axis Profile");
run("Find Peaks", "min._peak_amplitude=12.50 min._peak_distance=0 min._value=[] max._value=[] list");

segEnds=newArray((Table.get("X2", 0, title)),((Table.get("X2", 1, title))));
Array.sort(segEnds);

selectWindow(title);
run("Close");
selectWindow(expName+"-0-0");
close();
selectWindow("Peaks in "+expName+"-0-0");
close();
setBatchMode(true);

//extract segments based on black frame locations. Name segments from suer entered values in the initial dialog.
print("Extracting Segments");

selectWindow(expName);
print("Extracting "+stimOne+" Segment - Planes "+segSkip+" to "+(segSkip+segWidth-1));
run("Duplicate...", "title=["+stimOne+"] duplicate range="+segSkip+"-"+(segSkip+segWidth-1));

selectWindow(expName);
print("Extracting "+stimTwo+" Segment - Planes "+(segEnds[0]+segSkip)+" to "+(segEnds[0]+(segSkip+segWidth-1)));
run("Duplicate...", "title=["+stimTwo+"] duplicate range="+(segEnds[0]+segSkip)+"-"+(segEnds[0]+(segSkip+segWidth-1)));

selectWindow(expName);
print("Extracting "+stimThree+" Segment - Planes "+(segEnds[1]+segSkip)+" to "+(segEnds[1]+(segSkip+segWidth-1)));
run("Duplicate...", "title=["+stimThree+"] duplicate range="+(segEnds[1]+segSkip)+"-"+(segEnds[1]+(segSkip+segWidth-1)));

selectWindow(expName);
close();

for(s=0;s<stimList.length;s++){
	print("Analysing "+stimList[s]+" Segment");

	//select window from stimList array and store its dimensions
	selectWindow(stimList[s]);
	getDimensions(width, height, channels, slices, frames);

	//scale image to user defined scale
	run("Scale...", "x=- y=- z=1.0 width="+(width/scaleNumber)+" height="+(height/scaleNumber)+" depth="+(segWidth)+" interpolation=None process create title=Scaled");
	selectWindow("Scaled");

	//load tissue outline ROI and use it generate a standard deviation projection only of the tissue specific pixels 
	run("Set... ", "zoom=1600 x="+((width/scaleNumber)/2)+" y="+((height/scaleNumber)/2));
	roiManager("Open", roiDir+expName+" - Tissue Outline.roi");
	roiManager("Select",0);
		
	run("Z Project...", "projection=[Standard Deviation]");
	selectWindow("STD_Scaled");
	run("Restore Selection");
	run("Make Inverse");
	setColor(0);
	fill();
	run("Select None");
	run("Rainbow RGB");
	run("Set... ", "zoom=2000 x=16 y=16");
	run("Measure");
	maxInt=getResult("Max");
	minInt=getResult("Min");
	setThreshold(0.1, 10000);
	
	//if first time point store mean standard deviation value for whole section in stdDev variable
	run("Measure");
	if(s==0){
		stdDev=getResult("Mean");
	}
	
	resetThreshold;

	roiManager("reset");

	//work out how many boxes need to be created to cover entire image with 1x1 ROIs
	boxCountWidth=width/scaleNumber;
	boxCountHeight=height/scaleNumber;
	
	for(c=0;c<boxCountHeight;c++){
	for(b=0;b<boxCountWidth;b++){
		run("Specify...", "width=1 height=1 x="+b+" y="+c);
		roiManager("Add");
	
	}}

	//if first time point save generated ROI grid for later use
	if(s==0){
		roiManager("Save", roiDir+expName+" - ROI Grid.zip");
	}

	//generate arrays to store deviation counts
	stdDevArray=newArray();
	greaterThan2Std=newArray();
	between1and2Std=newArray();
	lessThan1Std=newArray();
	
	numROIs=roiManager("count");

	//process each ROI measuring its std dev and placing it into the relevant array
	for(r=0;r<numROIs;r++){
		roiManager("select", r);
		run("Measure");
		mean=getResult("Mean");
		if(mean>0){
		stdDevArray=Array.concat(stdDevArray, mean);
			if(mean>(stdDev*2)){
				greaterThan2Std=Array.concat(greaterThan2Std, mean);
			}
			if(mean<(stdDev*2)){
				if(mean>stdDev){
					between1and2Std=Array.concat(between1and2Std, mean);
				}
				else{
					lessThan1Std=Array.concat(lessThan1Std, mean);
				}
			}
			}
			}

	//print all meassured std devs into table
	for(a=0;a<stdDevArray.length;a++){
		print(tableTitle, expName+"\t"+stimList[s]+"\t"+scaleNumber+"\t"+(a+1)+"\t"+stdDevArray[a]);
	}
	
	//calculate and log counts and percentages of std dev shifts for whole tissue
	lengthA=stdDevArray.length;
	lengthB=greaterThan2Std.length;
	lengthC=between1and2Std.length;
	lengthD=lessThan1Std.length;
	
	print(tableTitle2, expName+"\t"+stimList[s]+"\t"+scaleNumber+"\t"+lengthA+"\t"+lengthB+"\t"+lengthC+"\t"+lengthD+"\t"+((lengthB/lengthA)*100)+"\t"+((lengthC/lengthA)*100)+"\t"+((lengthD/lengthA)*100));

	roiManager("reset");
	run("Select None");
	
	selectWindow("STD_Scaled");
	run("Select None");
	run("Remove Overlay");

	run("Scale...", "x=- y=- z=1.0 width="+(width)+" height="+(height)+" depth=1 interpolation=None process create title=["+stimList[s]+" - Heat Map]");

	selectWindow("STD_Scaled");
	saveAs("Tiff", heatMapDir+expName+" - "+stimList[s]+" - Scale Value "+scaleNumber+" - Raw Heat Map.tif");
	close();

	selectWindow("Scaled");
	close();

	print("Generating "+stimList[s]+" Heat Map Movie");
	
	selectWindow(stimList[s]);
	nslices=nSlices;
	getDimensions(swidth, sheight, schannels, sslices, sframes);
	run("Gaussian Blur...", "sigma=4 stack");
	//setBatchMode(true);
	newImage("Temp", "32-bit black", swidth, sheight, 1);
	
	averageFrameSize=windowSize;
	
	for(t=0;t<sframes;t++){
		if(t<(sframes-averageFrameSize)){
		selectWindow(stimList[s]);
		setSlice(t+1);
		run("Duplicate...", "title=[Temp Stack] duplicate range="+(t+1)+"-"+(t+averageFrameSize+1));
		selectWindow("Temp Stack");
		run("Z Project...", "projection=[Standard Deviation]");
		selectWindow("STD_Temp Stack");
		run("Select All");
		run("Copy");
		selectWindow("Temp");
		run("Add Slice");
		run("Paste");
		run("Select None");
		selectWindow("STD_Temp Stack");
		close();
		selectWindow("Temp Stack");
		close();
			
	}}
	selectWindow("Temp");
		run("Gaussian Blur...", "sigma=5 stack");
	run("Enhance Contrast", "saturated=0.05 use");
	run("Rainbow RGB");
	setSlice(1);
	run("Delete Slice");
	nslices=nSlices;
	run("Select None");
	saveAs("Tiff", movieDir+expName+" - "+stimList[s]+" - Rolling Average of "+windowSize+" Standard Deviations - Heat Map Movie");
	close();
	selectWindow(stimList[s]);
	close();
	
}

print("Generating Heat Map Montage");

run("Combine...", "stack1=["+stimList[0]+" - Heat Map] stack2=["+stimList[1]+" - Heat Map]");
selectWindow("Combined Stacks");
rename("Temp");
run("Combine...", "stack1=Temp stack2=["+stimList[2]+" - Heat Map]");
selectWindow("Combined Stacks");
run("Enhance Contrast", "saturated=0.5");
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - Heat Map Montage.tif");
run("Close All");

print("Saving Data Tables");

selectWindow("Raw Standard Deviation Values");
saveAs("Text", dataDir+expName+" - Scale Value "+scaleNumber+" - Raw Std Dev.csv");
run("Close");
selectWindow("Standard Deviation Scoring");
saveAs("Text", dataDir+expName+" - Scale Value "+scaleNumber+" - Std Dev Scoring.csv");
run("Close");

open(heatMapDir+expName+" - "+stimList[0]+" - Scale Value "+scaleNumber+" - Raw Heat Map.tif");
rename(stimList[0]);

open(heatMapDir+expName+" - "+stimList[1]+" - Scale Value "+scaleNumber+" - Raw Heat Map.tif");
rename(stimList[1]);

open(heatMapDir+expName+" - "+stimList[2]+" - Scale Value "+scaleNumber+" - Raw Heat Map.tif");
rename(stimList[2]);

imageCalculator("Divide create 32-bit", stimList[1],stimList[0]);
selectWindow("Result of "+stimList[1]);
rename("Fold Increase "+stimList[1]+" over "+stimList[0]);
oneName=getTitle();
roiManager("Open", roiDir+expName+" - ROI Grid.zip");

imageCalculator("Divide create 32-bit", stimList[2],stimList[0]);
selectWindow("Result of "+stimList[2]);
rename("Fold Increase "+stimList[2]+" over "+stimList[0]);
twoName=getTitle();
run("Show All");

imageCalculator("Divide create 32-bit", stimList[2],stimList[1]);
selectWindow("Result of "+stimList[2]);
rename("Fold Increase "+stimList[2]+" over "+stimList[1]);
threeName=getTitle();
run("Show All");

selectWindow(stimList[0]);
close();
selectWindow(stimList[1]);
close();
selectWindow(stimList[2]);
close();

foldArrayOne=newArray();
foldArrayTwo=newArray();
foldArrayThree=newArray();
upDownArrayBase=newArray();
upDownArraySequential=newArray();

numROIs=roiManager("count");

	
for(r=0;r<numROIs;r++){
	selectWindow(oneName);
	roiManager("select", r);
	run("Measure");
	mean=getResult("Mean");
	if(mean>0){
		selectWindow(oneName);
		roiManager("select", r);
		run("Measure");
		oneMean=getResult("Mean");
		foldArrayOne=Array.concat(foldArrayOne, oneMean);

		selectWindow(twoName);
		roiManager("select", r);
		run("Measure");
		twoMean=getResult("Mean");
		foldArrayTwo=Array.concat(foldArrayTwo, twoMean);

		selectWindow(threeName);
		roiManager("select", r);
		run("Measure");
		threeMean=getResult("Mean");
		foldArrayThree=Array.concat(foldArrayThree, threeMean);		

		toStartDirectionString="";
		toPreviousDirectionString="";

		if(oneMean>twoMean){
			toStartDirectionString=toStartDirectionString+"0";
			toPreviousDirectionString=toPreviousDirectionString+"0";
		}else{
			toStartDirectionString=toStartDirectionString+"1";
			toPreviousDirectionString=toPreviousDirectionString+"1";
		}
		if(oneMean>threeMean){
			toStartDirectionString=toStartDirectionString+"0";
		}else{
			toStartDirectionString=toStartDirectionString+"1";
		}
		if(twoMean>threeMean){
			toPreviousDirectionString=toPreviousDirectionString+"0";
		}else{
			toPreviousDirectionString=toPreviousDirectionString+"1";
		}

		upDownArrayBase=Array.concat(upDownArrayBase, toStartDirectionString);
		upDownArraySequential=Array.concat(upDownArraySequential, toPreviousDirectionString);
		
		
		}
		}
	for(p=0;p<foldArrayOne.length;p++){
		print(tableTitle3, expName+"\t"+scaleNumber+"\t"+(p+1)+"\t"+foldArrayOne[p]+"\t"+foldArrayTwo[p]+"\t"+foldArrayThree[p]+"\t"+upDownArrayBase[p]+"\t"+upDownArraySequential[p]);
	}


selectWindow("Results");
run("Close");
selectWindow("ROI Manager");
run("Close");


print("Generating Heat Relative Map Montage");

selectWindow(oneName);
run("Select None");
run("Remove Overlay");
run("Duplicate...", "title=Temp1");
selectWindow(twoName);
run("Select None");
run("Remove Overlay");
run("Duplicate...", "title=Temp2");
selectWindow(threeName);
run("Select None");
run("Remove Overlay");
run("Duplicate...", "title=Temp3");

run("Combine...", "stack1=[Temp1] stack2=[Temp2]");
selectWindow("Combined Stacks");
rename("Temp");
run("Combine...", "stack1=Temp stack2=[Temp3]");
selectWindow("Combined Stacks");
run("Enhance Contrast", "saturated=0.5");
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - Relvative Change Heat Map Montage.tif");
close();

selectWindow(oneName);
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - "+oneName+".tif");
close();

selectWindow(twoName);
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - "+twoName+".tif");
close();

selectWindow(threeName);
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - "+threeName+".tif");
close();

selectWindow("Deviation Relative Change");
saveAs("Text", dataDir+expName+" - Scale Value "+scaleNumber+" - Relative Std Dev.csv");
run("Close");

foldArrayOneRolling=newArray();
foldArrayTwoRolling=newArray();
foldArrayThreeRolling=newArray();
upDownArrayBaseRolling=newArray();
upDownArraySequentialRolling=newArray();

for(n=0;n<stimList.length;n++){
	fileName=movieDir+expName+" - "+stimList[n]+" - Rolling Average of "+windowSize+" Standard Deviations - Heat Map Movie.tif";
	open(fileName);
	nameStore=getTitle();
	run("Z Project...", "projection=[Standard Deviation]");
	run("Scale...", "x=- y=- z=1.0 width="+(width/scaleNumber)+" height="+(height/scaleNumber)+" interpolation=None process create title=["+stimList[n]+"] - Rolling");
	roiManager("reset");
	selectWindow(stimList[n]);
	roiManager("Open", roiDir+expName+" - Tissue Outline.roi");
	roiManager("Select",0);
	run("Make Inverse");
	setColor(0);
	fill();
	run("Select None");
	run("Remove Overlay");
	run("Rainbow RGB");
	run("Set... ", "zoom=2000 x=16 y=16");
	selectWindow(nameStore);
	close();
	roiManager("reset");
	selectWindow("STD_"+nameStore);
	close();
}

imageCalculator("Divide create 32-bit", stimList[1],stimList[0]);
selectWindow("Result of "+stimList[1]);
rename("Fold Increase "+stimList[1]+" over "+stimList[0]+" - Rolling");
oneName=getTitle();
roiManager("Open", roiDir+expName+" - ROI Grid.zip");

imageCalculator("Divide create 32-bit", stimList[2],stimList[0]);
selectWindow("Result of "+stimList[2]);
rename("Fold Increase "+stimList[2]+" over "+stimList[0]+" - Rolling");
twoName=getTitle();
run("Show All");

imageCalculator("Divide create 32-bit", stimList[2],stimList[1]);
selectWindow("Result of "+stimList[2]);
rename("Fold Increase "+stimList[2]+" over "+stimList[1]+" - Rolling");
threeName=getTitle();
run("Show All");

selectWindow(stimList[0]);
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - "+stimList[0]+" - Rolling.tif");
close();
selectWindow(stimList[1]);
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - "+stimList[1]+" - Rolling.tif");
close();
selectWindow(stimList[2]);
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - "+stimList[2]+" - Rolling.tif");
close();

roiManager("Open", roiDir+expName+" - ROI Grid.zip");
		
	for(r=0;r<numROIs;r++){
		selectWindow(oneName);
		roiManager("select", r);
		run("Measure");
		mean=getResult("Mean");
		if(mean>0){
			selectWindow(oneName);
			roiManager("select", r);
			run("Measure");
			oneMean=getResult("Mean");
			foldArrayOneRolling=Array.concat(foldArrayOneRolling, oneMean);
	
			selectWindow(twoName);
			roiManager("select", r);
			run("Measure");
			twoMean=getResult("Mean");
			foldArrayTwoRolling=Array.concat(foldArrayTwoRolling, twoMean);
	
			selectWindow(threeName);
			roiManager("select", r);
			run("Measure");
			threeMean=getResult("Mean");
			foldArrayThreeRolling=Array.concat(foldArrayThreeRolling, threeMean);		
	
			toStartDirectionString="";
			toPreviousDirectionString="";
	
			if(oneMean>twoMean){
				toStartDirectionString=toStartDirectionString+"0";
				toPreviousDirectionString=toPreviousDirectionString+"0";
			}else{
				toStartDirectionString=toStartDirectionString+"1";
				toPreviousDirectionString=toPreviousDirectionString+"1";
			}
			if(oneMean>threeMean){
				toStartDirectionString=toStartDirectionString+"0";
			}else{
				toStartDirectionString=toStartDirectionString+"1";
			}
			if(twoMean>threeMean){
				toPreviousDirectionString=toPreviousDirectionString+"0";
			}else{
				toPreviousDirectionString=toPreviousDirectionString+"1";
			}
	
			upDownArrayBaseRolling=Array.concat(upDownArrayBaseRolling, toStartDirectionString);
			upDownArraySequentialRolling=Array.concat(upDownArraySequentialRolling, toPreviousDirectionString);
			
			
			}
			}
		for(p=0;p<foldArrayOneRolling.length;p++){
			print(tableTitle4, expName+"\t"+scaleNumber+"\t"+(p+1)+"\t"+foldArrayOneRolling[p]+"\t"+foldArrayTwoRolling[p]+"\t"+foldArrayThreeRolling[p]+"\t"+upDownArrayBaseRolling[p]+"\t"+upDownArraySequentialRolling[p]);
		}

selectWindow("Deviation Relative Change - Rolling Average");
saveAs("Text", dataDir+expName+" - Scale Value "+scaleNumber+" - Relative Std Dev - Rolling Average.csv");
run("Close");

selectWindow(oneName);
run("Select None");
run("Remove Overlay");
run("Duplicate...", "title=Temp1");
selectWindow(twoName);
run("Select None");
run("Remove Overlay");
run("Duplicate...", "title=Temp2");
selectWindow(threeName);
run("Select None");
run("Remove Overlay");
run("Duplicate...", "title=Temp3");

run("Combine...", "stack1=[Temp1] stack2=[Temp2]");
selectWindow("Combined Stacks");
rename("Temp");
run("Combine...", "stack1=Temp stack2=[Temp3]");
selectWindow("Combined Stacks");
run("Enhance Contrast", "saturated=0.5");
saveAs("Tiff", heatMapDir+expName+" - Scale Value "+scaleNumber+" - Relvative Change - Rolling Average - Heat Map Montage.tif");
close();

selectWindow("Results");
run("Close");
//selectWindow("ROI Manager");
//run("Close");

run("Close All");

	



setBatchMode("exit and display");
print("FINISHED!!!");





